/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#ifndef KOMASYNC_H
#define KOMASYNC_H

#include "komclient.h"

class KOMAsync
{
public:
	KOMAsync(KOMSession* kom) : komSession(kom), unreadUpdate(0), newTexts(false), nameUpdate(0), lastMessage(NULL), lastEventId(1), needsClearCache(false)
	{}
	~KOMAsync();
	bool progress(KOMString& reply);

	/** Get the events which was received since last poll. */
	int getEvents(KOMString& evt, int prevId);
	void updateUnread();
	bool hasNewTexts(){return newTexts;}

	/** Add a new instant message to the instant message list. */
	void addInstantMessage(int from, int to, KOMString& text);
	/** Mark an instant message as read. */
	void markInstantMessageRead(int eventId);
	bool hasUnreadInstantMsg();

	void clearCacheIfNeeded();
private:
	/** Update a conference name. */
	void updateConfName(KOMString& str);
	/** Add a new text to all apropriate caches. 
	 * @returns the global number of the added text. */
	int addNewText(KOMString& str);
	KOMSession* komSession;
	struct AsyncMessage
	{
		AsyncMessage()
		{}
		~AsyncMessage()
		{}
		KOMString text;
		int to;
		int from;
		int eventId;
		bool read;
		time_t timestamp;
		AsyncMessage* next;
		AsyncMessage* prev;
	};
	int unreadUpdate;
	bool newTexts;
	int nameUpdate;
	AsyncMessage* lastMessage;
	int lastEventId;
	bool needsClearCache;
};

#endif // KOMASYNC_H

