/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include "komchangepresentation.h"
#include "version.h"

const char* const KOMChangePresentationCommand::commandString = "change_presentation";

int KOMChangePresentationCommand::Construct(KOMString& cmd)
{
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confNum) || !cmd.parseCommandString(presentation))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	KOMString cmdstr;
	// Get conf stat to check if there is an old presentation
	cmdID = komSession->getCommandID();
	if (presentation.len() == 0)
	{
		// Remove the presentation
		state = KOM_PRES_SET_PRES;
		if (cmdstr.appendFormat("%d 16 %d 0\n", cmdID, confNum))
		{
			error(KOM_ERROR_OOM);
			return 0;
		}
	}
	else
	{
		state = KOM_PRES_CHECK_OLD;
		if (cmdstr.appendFormat("%d 91 %d\n", cmdID, confNum))
		{
			error(KOM_ERROR_OOM);
			return 0;
		}
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMChangePresentationCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
	{
		if (state == KOM_PRES_CHECK_OLD)
		{
			bool status;
			unsigned int type;
			time_t t;
			status = reply.parseString(name);
			if (status)
				status = reply.parseBitfield(type);
			if (status)
				status = reply.parseTime(t);
			if (status)
				status = reply.parseTime(t);
			if (status)
				status = reply.parseInt(oldTextNum);
			if (status)
				status = reply.parseInt(oldTextNum);
			if (!status)
			{
				error(KOM_ERROR_SYNTAX);
				return true;
			}
			person = (type&16)!=0;

			KOMString cmdstr;
			cmdID = komSession->getCommandID();
			if (cmdstr.appendFormat("%d 94\n", cmdID))
			{
				error(KOM_ERROR_OOM);
				return true;
			}
			state = KOM_PRES_CHECK_CONF;
			komSession->sendCommand(cmdstr.c_string());
		}
		else if (state == KOM_PRES_CHECK_CONF)
		{
			bool status = reply.parseInt(presentationConf);
			if (status)
				status = reply.parseInt(presentationConf);
			if (status && person)
				status = reply.parseInt(presentationConf);
			if (!status)
			{
				error(KOM_ERROR_SYNTAX);
				return true;
			}
			
			KOMString cmdstr;
			cmdID = komSession->getCommandID();

			KOMString auxArray;
			KOMString version;
			version.appendFormat("KOMclean %s", KOMCLEAN_VERSION_STRING);
			auxArray.appendFormat("2 { 15 00000 1 %dH%s 1 00000 1 ", version.len(), version.c_string());
			if (!presentation.UTF8ToLatin1())
			{
				auxArray.appendFormat("%dH%s }", KOM_CONTENT_TYPE_UTF8_LEN, KOM_CONTENT_TYPE_UTF8);
			}
			else
			{
				auxArray.appendFormat("%dH%s }", KOM_CONTENT_TYPE_LEN, KOM_CONTENT_TYPE);
			}

			int err = cmdstr.appendFormat("%d 86 %dH%s\n%s ", cmdID, 
					name.len()+presentation.len()+1,
					name.c_string(), presentation.c_string());
			if (!err && oldTextNum > 0)
				err = cmdstr.appendFormat("2 { 0 %d 2 %d } %s\n", presentationConf, oldTextNum, auxArray.c_string());
			else if (!err)
				err = cmdstr.appendFormat("1 { 0 %d } 0 { }\n", presentationConf);
			if (err)
			{
				error(KOM_ERROR_OOM);
				return true;
			}
			state = KOM_PRES_CREATE_TEXT;
			komSession->sendCommand(cmdstr.c_string());
		}
		else if (state == KOM_PRES_CREATE_TEXT)
		{
			int textNum;
			if (!reply.parseInt(textNum))
			{
				error(KOM_ERROR_SYNTAX);
				return true;
			}
			
			KOMString cmdstr;
			cmdID = komSession->getCommandID();
			if (cmdstr.appendFormat("%d 16 %d %d\n", cmdID, confNum, textNum))
			{
				error(KOM_ERROR_OOM);
				return true;
			}
			state = KOM_PRES_SET_PRES;
			komSession->sendCommand(cmdstr.c_string());
		}
		else
		{
			ok("");
		}
	}
	return true;
}

