/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include <stdlib.h>
#include "komcreateconf.h"

const char* const KOMCreateConferenceCommand::commandString = "create_conference";
const char* const KOMChangeConferenceTypeCommand::commandString = "change_conference_type";
const char* const KOMRenameConferenceCommand::commandString = "rename_conference";
const char* const KOMChangeSupervisorCommand::commandString = "change_supervisor";

int KOMCreateConferenceCommand::Construct(KOMString& cmd)
{
	KOMString name;
	if (!cmd.parseLiteral(commandString) || !cmd.parseCommandString(name))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	if (!name.UTF8ToLatin1())
	{
		error(KOM_ERROR_INVALID_CHARACTERS);
		return 0;
	}
	
	cmdID = komSession->getCommandID();

	KOMString cmdstr;
	if (cmdstr.appendFormat("%d 88 %dH%s 00000000 0 { }\n", cmdID, name.len(), name.c_string()))
	{
		error(KOM_ERROR_OOM);
		return 0;
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMCreateConferenceCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
		ok("");
	return true;
}

int KOMChangeConferenceTypeCommand::Construct(KOMString& cmd)
{
	int confNum, prot, original, secret, anonok, nosecret;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confNum) || 
		!cmd.parseInt(prot) || !cmd.parseInt(original) || 
		!cmd.parseInt(secret) || !cmd.parseInt(anonok) || 
		!cmd.parseInt(nosecret))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	
	cmdID = komSession->getCommandID();

	KOMString cmdstr;
	// FIXME: this means it is only possible to change the type of a conference,
	// not a users inbox
	if (cmdstr.appendFormat("%d 21 %d %d%d%d0%d%d00\n", cmdID, confNum, 
			prot?1:0, original?1:0, secret?1:0, anonok?1:0, nosecret?1:0))
	{
		error(KOM_ERROR_OOM);
		return 0;
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMChangeConferenceTypeCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
		ok("");
	return true;
}

int KOMRenameConferenceCommand::Construct(KOMString& cmd)
{
	int confNum;
	KOMString name;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confNum) ||
			!cmd.parseCommandString(name))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	if (!name.UTF8ToLatin1())
	{
		error(KOM_ERROR_INVALID_CHARACTERS);
		return 0;
	}
	
	cmdID = komSession->getCommandID();

	KOMString cmdstr;
	if (cmdstr.appendFormat("%d 3 %d %dH%s\n", cmdID, confNum, name.len(), name.c_string()))
	{
		error(KOM_ERROR_OOM);
		return 0;
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMRenameConferenceCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
		ok("");
	return true;
}

int KOMChangeSupervisorCommand::Construct(KOMString& cmd)
{
	int confNum, persNum;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confNum) ||
			!cmd.parseInt(persNum))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	
	cmdID = komSession->getCommandID();

	KOMString cmdstr;
	if (cmdstr.appendFormat("%d 18 %d %d\n", cmdID, confNum, persNum))
	{
		error(KOM_ERROR_OOM);
		return 0;
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMChangeSupervisorCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
		ok("");
	return true;
}


