/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include <stdlib.h>
#include "kommarkread.h"

const char* const KOMMarkReadCommand::commandString = "mark_read";
const char* const KOMIgnoreThreadCommand::commandString = "ignore_thread";

int KOMMarkReadCommand::Construct(KOMString& cmd)
{
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confNum))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	mark_all = false;
	
	KOMConferenceStat* c = komSession->getConference(confNum);
	KOMString cmdstr;

	cmdID = -1;
	pendingCmds = NULL;

	if (!cmd.parseInt(textNum))
	{
		mark_all = true;
		textNum = c->getHighestLocal();
	}

	if (!mark_all)
	{
		KOMTextStat* txt = NULL;
		txt = komSession->getGlobalTextStat(textNum);
		if (!txt || txt->getLocalNum(confNum) <= 0)
		{
			error(KOM_ERROR_PROTOCOL);
			return 0;
		}
		for (c = komSession->getFirstConference(); c; c = c->nextConference())
		{
			if (txt->getLocalNum(c->getNum()))
			{
				MarkConfCmd* cmd = new MarkConfCmd();
				if (!cmd)
				{
					while (pendingCmds)
					{
						cmd = pendingCmds->next;
						delete pendingCmds;
						pendingCmds = cmd;
					}
					error(KOM_ERROR_OOM);
					return 0;
				}
				cmd->cmdID = komSession->getCommandID();
				cmd->next = pendingCmds;
				cmd->confNum = c->getNum();
				cmd->textNum = txt->getLocalNum(c->getNum());
				pendingCmds = cmd;

				if (cmdstr.appendFormat("%d 27 %d 1 { %d }\n", cmd->cmdID, cmd->confNum, cmd->textNum))
				{
					while (pendingCmds)
					{
						cmd = pendingCmds->next;
						delete pendingCmds;
						pendingCmds = cmd;
					}
					error(KOM_ERROR_OOM);
					return 0;
				}
			}
		}
	}
	else
	{
		cmdID = komSession->getCommandID();

		if (cmdstr.appendFormat("%d 77 %d %d\n", cmdID, confNum, textNum))
		{
			error(KOM_ERROR_OOM);
			return 0;
		}
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMMarkReadCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	MarkConfCmd* cmd = pendingCmds;
	while (cmd && cmd->cmdID != id)
		cmd = cmd->next;
	if (id != cmdID && !cmd)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
	{
		if (cmd)
		{
			confNum = cmd->confNum;
			textNum = cmd->textNum;
			MarkConfCmd** nextp = &pendingCmds;
			while (*nextp != cmd)
				nextp = &(*nextp)->next;
			*nextp = (*nextp)->next;
			delete cmd;
		}
		KOMConferenceStat* c = komSession->getConference(confNum);
		if (c)
		{
			if (mark_all)
			{
				for (int i = 1; i <= textNum; ++i)
					c->addReadText(i);
			}
			else
				c->addReadText(textNum);
		}
		if (!pendingCmds)
		{
			komSession->updateUnread();
			ok("");
		}
	}
	return true;
}

int KOMIgnoreThreadCommand::Construct(KOMString& cmd)
{
	int textNum;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(textNum))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	komSession->addIgnored(textNum);
	ok("");
	return 0;
}

