/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include <stdlib.h>
#include "komnextunreadthread.h"

const char* const KOMNextUnreadThreadCommand::commandString = "next_unread_thread";

KOMNextUnreadThreadCommand::KOMNextUnreadThreadCommand(KOMCommandListener* l, int sid, KOMSession* kom) : KOMCommand(l, sid, kom), conf(NULL)
{}

KOMNextUnreadThreadCommand::~KOMNextUnreadThreadCommand()
{
	if (conf)
	{
		conf->decRef();
	}
}


int KOMNextUnreadThreadCommand::Construct(KOMString& cmd)
{
	if (!cmd.parseLiteral(commandString))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	conf = komSession->getFirstConference();
	while (conf && conf->isRead())
		conf = conf->nextConference();
	if (!conf)
	{
		ok("");
		return 0;
	}
	conf->addRef();

	KOMTextStat* txt = NULL;
	textNum = conf->getHighestLocal()+1;
	while ((!txt || txt->isUnavailable()) && textNum > 0)
	{
		textNum = conf->findNextLowerUnread(textNum-1);
		txt = komSession->getGlobalTextStat(conf->getGlobalTextNum(textNum));
	}
	if (!txt || txt->isUnavailable())
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	textNum = txt->getGlobalNum();
	prevTextNum = -1;
#if 1
	// get thread will now find the root, so there is no need to do it here
	KOMString re;
	re.appendFormat(KOM_GROUP_SEPARATOR "%d" KOM_RECORD_SEPARATOR "%d", conf->getNum(), textNum);
	ok(re.c_string());
#else
	findThreadRoot();
#endif
	return 0;
}

void KOMNextUnreadThreadCommand::findThreadRoot()
{
	KOMTextStat* txt = komSession->getGlobalTextStat(textNum);
	while (txt && !txt->isUnavailable() && txt->hasStat() && txt->getCommentTo() > 0)
	{
		prevTextNum = textNum;
		textNum = txt->getCommentTo();
		txt = komSession->getGlobalTextStat(textNum);
	}
	if (txt && txt->isUnavailable())
	{
		// prevtext is the text root
		textNum = prevTextNum;
		txt = komSession->getGlobalTextStat(textNum);
	}
	if (!txt || !txt->hasStat())
	{
		// request the next text
		cmdId = komSession->getCommandID();
		// FIXME: OOM
		KOMString cmdstr;
		cmdstr.appendFormat("%d 90 %d\n", cmdId, textNum);
		komSession->sendCommand(cmdstr.c_string());
		return;
	}
	KOMString re;
	re.appendFormat(KOM_GROUP_SEPARATOR "%d" KOM_RECORD_SEPARATOR "%d", conf->getNum(), textNum);
	ok(re.c_string());
}

bool KOMNextUnreadThreadCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdId)
		return false;
	else
	{
		int status = komSession->addTextStat(textNum, is_ok, reply);
		if (status > 0)
		{
			error(KOM_ERROR_PROTOCOL);
			return true;
		}
		else if (status < 0)
		{
			error(KOM_ERROR_OOM);
			return true;
		}
		findThreadRoot();
	}
	return true;
}

