/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include <stdlib.h>
#include "komsendinstantmsg.h"

const char* const KOMSendInstantMsgCommand::commandString = "send_instant_message";

int KOMSendInstantMsgCommand::Construct(KOMString& cmd)
{
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(toNum))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	if (!cmd.parseCommandString(text))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	if (!text.UTF8ToLatin1())
	{
		error(KOM_ERROR_INVALID_CHARACTERS);
		return 0;
	}
	
	cmdID = komSession->getCommandID();

	KOMString cmdstr;
	if (cmdstr.appendFormat("%d 53 %d %dH%s", cmdID, toNum, text.len(), text.c_string()))
	{
		error(KOM_ERROR_OOM);
		return 0;
	}

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMSendInstantMsgCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
	{
		// If message is sent to a conference or user which the current
		// user is not subscribed to, add the message to the log
		if (!komSession->getConference(toNum))
		{
			text.latin1ToUTF8();
			komSession->addInstantMessage(komSession->getUserID(), toNum, text);
		}
		ok("");
	}
	return true;
}

