#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "komsubscribe.h"

const char* const KOMSubscribeCommand::commandString = "subscribe";
const char* const KOMChangeSubscriptionTypeCommand::commandString = "change_subscription_type";

int KOMSubscribeCommand::Construct(KOMString& cmd)
{
	KOMString cmdstr;
	int subscribe;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(subscribe) || !cmd.parseInt(userID) || !cmd.parseInt(confID))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	if (userID < 0)
		userID = komSession->getUserID();
	cmdID = komSession->getCommandID();
	// FIXME: OOM
	if (subscribe)
		cmdstr.appendFormat("%d 100 %d %d 100 0 00000000\n", cmdID, confID, userID);
	else
		cmdstr.appendFormat("%d 15 %d %d\n", cmdID, confID, userID);

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMSubscribeCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
		ok("");
	return true;
}

int KOMChangeSubscriptionTypeCommand::Construct(KOMString& cmd)
{
	KOMString cmdstr;
	if (!cmd.parseLiteral(commandString) || !cmd.parseInt(confID) || !cmd.parseInt(secret) || !cmd.parseInt(passive) || !cmd.parseInt(instant_passive))
	{
		error(KOM_ERROR_SYNTAX);
		return 0;
	}
	cmdID = komSession->getCommandID();
	// FIXME: OOM
	cmdstr.appendFormat("%d 102 %d %d 0%d%d%d0000\n", cmdID, komSession->getUserID(), confID, passive?1:0, secret?1:0, (instant_passive^passive)?1:0);

	komSession->sendCommand(cmdstr.c_string());
	return 0;
}

bool KOMChangeSubscriptionTypeCommand::progress(KOMString& reply)
{
	bool is_ok;
	int id;
	if (!reply.parseStatus(is_ok) || !reply.parseInt(id))
		return false;
	if (id != cmdID)
		return false;
	if (!is_ok)
		error(reply.parseErrorCode());
	else
	{
		KOMConferenceStat* c = komSession->getConference(confID);
		if (c)
		{
			c->setInvite(false);
			c->setSecret(secret!=0);
			c->setPassive(passive!=0, instant_passive!=0);
		}
		ok("");
	}
	return true;
}

