/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2008 Tim Johansson.  All rights reserved.
**
*/

#include "komui.h"

#ifndef WIN32
#include <stdlib.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#ifndef MAX_PATH
#define MAX_PATH 1024
#endif // MAX_PATH
bool GetInstallPath(const char* cmd, const char* project, char** dataPath)
{
	char tempPath[MAX_PATH+1];
	if (strlen(cmd) > MAX_PATH)
		return false;
	if (strchr(cmd, '/'))
	{
		// Path contains a slash, that means a relative or absolute path
		strcpy(tempPath, cmd);
	}
	else
	{
		char* path = getenv("PATH");
		if (!path)
			return false;
		while(path)
		{
			char* nextPath = strchr(path, ':');
			unsigned int pathLen;
			if (nextPath)
				pathLen = ptrdiff_t(nextPath-path);
			else
				pathLen = strlen(path);
			if (pathLen)
			{
				if (pathLen > MAX_PATH)
					return false;
				memcpy(tempPath, path, pathLen);
				tempPath[pathLen] = '\0';
				if (tempPath[pathLen-1] != '/')
				{
					strncat(tempPath, "/", MAX_PATH-pathLen);
					++pathLen;
				}
				strncat(tempPath, cmd, MAX_PATH-pathLen);
				struct stat buf;
				if (!stat(tempPath, &buf))
					break;
			}
			path = nextPath;
			if (path)
				++path;
		}
		// Not found in path
		if (!path)
			return false;
	}
	if (tempPath[0] != '/')
	{
		char tempCmd[MAX_PATH+1];
		strcpy(tempCmd, tempPath);
		// Relative path
		if (!getcwd(tempPath, MAX_PATH))
			return false;
		if (tempPath[strlen(tempPath)-1] != '/')
			strncat(tempPath, "/", MAX_PATH-strlen(tempPath));
		strncat(tempPath, tempCmd, MAX_PATH-strlen(tempPath));
	}
	// tempPath now contains the path to the executable
	// strip all /./ and /../
	char* invalidPart;
	while ((invalidPart = strstr(tempPath, "/./")))
	{
		++invalidPart;
		while (invalidPart[2])
		{
			invalidPart[0] = invalidPart[2];
			++invalidPart;
		}
		*invalidPart = '\0';
	}
	while ((invalidPart = strstr(tempPath, "/../")))
	{
		// If invalidPart is tempPath the path starts with .., just remove it
		if (invalidPart == tempPath)
		{
			++invalidPart;
			while (invalidPart[3])
			{
				invalidPart[0] = invalidPart[3];
				++invalidPart;
			}
			*invalidPart = '\0';
		}
		else
		{
			char* invalidStart = invalidPart-1;
			while (invalidStart != tempPath && *invalidStart != '/')
				--invalidStart;
			invalidPart += 3;
			while (*invalidPart)
			{
				*invalidStart = *invalidPart;
				++invalidPart;
				++invalidStart;
			}
			*invalidStart = '\0';
		}
	}
	// TODO: follow all sym links
	// lstat + readlink
	// Add space so we can replace /bin with /share/project
	*dataPath = new char[strlen(tempPath)+strlen(project)+4];
	if (!*dataPath)
		return false;
	strcpy(*dataPath, tempPath);
	char* lastPart = strrchr(*dataPath, '/');
	if (!lastPart)
		return false;
	*lastPart = '\0';
	lastPart = strrchr(*dataPath, '/');
	if (lastPart && !strcmp(lastPart, "/bin"))
	{
		strcpy(lastPart, "/share/");
		strcat(lastPart, project);
	}
#ifdef MACOSX
	else
	{
		// There is already space so we can replace /MacOS with /Resources since it can't be done if last substitution succeded
		if (lastPart && !strcmp(lastPart, "/MacOS"))
		{
			strcpy(lastPart, "/Resources");
		}
	}
#endif // MACOSX
	return true;
}
#endif // !WIN32

#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <winsock.h>
#include <stdlib.h>
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
	WORD wVersionRequested;
	WSADATA wsaData;
	int err;

	wVersionRequested = MAKEWORD( 2, 2 );

	err = WSAStartup( wVersionRequested, &wsaData );
	if (err != 0)
		return 1;

	if (LOBYTE(wsaData.wVersion) != 2 ||
		HIBYTE(wsaData.wVersion) != 2)
	{
		WSACleanup();
		return 1; 
	}

	char dataPath[MAX_PATH];
	GetModuleFileNameA(NULL, dataPath, MAX_PATH);
	char* lastPos = strrchr(dataPath, '\\');
	if (lastPos)
	{
		if (strchr(lastPos, '/'))
			lastPos = strchr(lastPos, '/');
	}
	else
		lastPos = strrchr(dataPath, '/');
	if (lastPos)
		*lastPos = '\0';
	int argc = __argc;
	char** argv = __argv;
#else // !WIN32
#ifdef MACOSX
extern "C" int MacMain(int argc, char** argv)
#else
int main(int argc, char** argv)
#endif
{
	char* dataPath = NULL;
	if (!GetInstallPath(argv[0], "komclean", &dataPath))
		return -1;
#endif // !WIN32

	KOMUi* ui;
	if (KOMUi::Create(&ui, dataPath, argc, argv))
		return -1;
	ui->run();
	delete ui;
#ifdef WIN32
	WSACleanup();
#else // !WIN32
	delete[] dataPath;
#endif // !WIN32
	return 0;
}

