/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2009 Tim Johansson.  All rights reserved.
**
*/

#include "mactrayicon.h"
#include <stdlib.h>
#include <string.h>

#import <Cocoa/Cocoa.h>

static int mac_pipe[2];
static int mac_argc;
static char** mac_argv;
extern "C" int MacMain(int argc, char** argv);

@interface MacMainApplication : NSObject
{
}
@end
@implementation MacMainApplication
- (void) applicationDidFinishLaunching:(NSNotification *)notification
{
	int foo = 0;
	write(mac_pipe[1], &foo, sizeof(int));
}
@end

@interface MacMainFunctionRunner : NSObject
{
}
@end
@implementation MacMainFunctionRunner
- (void) runMacMainThread:(void*)nada
{
	int foo;
	while (read(mac_pipe[0], &foo, sizeof(int)) != sizeof(int)) ;
	MacMain(mac_argc, mac_argv);
	[NSApp stop:self];
}
@end

static MacMainApplication* mainApplication = NULL;

int main(int argc, char** argv)
{
	mac_argc = argc;
	mac_argv = argv;
	pipe(mac_pipe);

    NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
    [NSApplication sharedApplication];

    MacMainFunctionRunner *mm = [[MacMainFunctionRunner alloc] init];
	[NSThread detachNewThreadSelector: @selector(runMacMainThread:)
		toTarget: mm withObject: nil];
   
    mainApplication = [[MacMainApplication alloc] init];
    [NSApp setDelegate:mainApplication];
    [NSApp run];
	
	[mm release];
	[mainApplication release];
	
	close(mac_pipe[0]);
	close(mac_pipe[1]);

	[pool release];
}

void SystemOpenURL(const char* url)
{
	NSString* urlstr = [NSString stringWithUTF8String:url]; 
	NSURL *nsurl = [NSURL URLWithString:urlstr];
	[[NSWorkspace sharedWorkspace] openURL:nsurl];
	[nsurl release];
	[urlstr release];
}

/////////////// TrayIcon implementation ///////////////

@interface MacStatusBar : NSObject
{
@private
	NSStatusItem* statusItem;
	NSMenu* menu;
}

@end

@implementation MacStatusBar

- (void) triggerMenuAction:(id)sender {
	write(mac_pipe[1], &sender, sizeof(sender));
}

- (void) createMenu {
	NSZone *menuZone = [NSMenu menuZone];
	menu = [[NSMenu allocWithZone:menuZone] init];
  
	statusItem = NULL;
}

- (void) show:(int)vis {
	if (vis)
	{
		statusItem = [[[NSStatusBar systemStatusBar]
                  statusItemWithLength:NSSquareStatusItemLength] retain];
		[statusItem setMenu:menu];
		[statusItem setHighlightMode:YES];
		[statusItem setToolTip:@"KOMclean"];
	}
	else if (statusItem)
	{
		// FIXME: implement hide
		assert(!"Hide not implemented");
	}
}

- (id) addMenuItem: (NSString*)title {
	NSMenuItem* menuItem;
	menuItem = [menu addItemWithTitle:title
                      action:@selector(triggerMenuAction:)
                      keyEquivalent:@""];
	[menuItem setTarget:self];
	return menuItem;
}

- (void) changeIcon: (NSString*)icn {
	[statusItem setImage:[NSImage imageNamed:icn]];
}

- (void) dealloc {
	if (statusItem)
		[statusItem release];
	[menu release];
	[super dealloc];
}
@end


MacTrayIcon::MacTrayIcon(TrayIconListener* l) : listener(l), popupMenu(NULL), icons(NULL), numIcons(0), activeIcon(0), isVisible(false)
{
	macStatusBar = [[MacStatusBar alloc] init];
	[macStatusBar createMenu];
}

MacTrayIcon::~MacTrayIcon()
{
	clearPopupMenu();
	for (int i = 0; i < numIcons; ++i)
		delete[] icons[i];
	delete[] icons;
	[macStatusBar release];
}

int MacTrayIcon::addIcon(void* icon)
{
	char** newicons;
	newicons = new char*[numIcons+1];
	if (!newicons)
		return -1;
	newicons[numIcons] = new char[strlen((char*)icon)+1];
	if (!newicons[numIcons])
	{
		delete[] newicons;
		return -1;
	}
	strcpy(newicons[numIcons], (char*)icon);
	for (int i = 0; i < numIcons; ++i)
		newicons[i] = icons[i];
	delete[] icons;
	icons = newicons;
	++numIcons;
	return 0;
}

int MacTrayIcon::setActiveIcon(int iconId)
{
	if (iconId < 0 || iconId >= numIcons)
		return -1;
	activeIcon = iconId;
	if (isVisible)
	{
		NSString* istr = [NSString stringWithUTF8String:icons[iconId]];
		[macStatusBar changeIcon:istr]; 
		[istr release];
	}
	return 0;
}

int MacTrayIcon::show(int vis)
{
	if (vis && !isVisible)
	{
		isVisible = true;
		[macStatusBar show:1];
		setActiveIcon(activeIcon);
	}
	else if (!vis && isVisible)
	{
		isVisible = false;
		[macStatusBar show:0];
	}
	return 0;
}

void MacTrayIcon::clearPopupMenu()
{
	// FIXME: actually remove the entries from the menu
	while (popupMenu)
	{
		TrayPopupMenuItem* nm = popupMenu->next;
		delete[] popupMenu->text;
		delete popupMenu;
		popupMenu = nm;
	}
}

int MacTrayIcon::addPopupMenuItem(const char* text, int id, bool def)
{
	TrayPopupMenuItem* item = new TrayPopupMenuItem;
	if (!item)
		return -1;
	item->next = NULL;
	item->id = id;
	item->text = new char[strlen(text)+1];
	if (!item->text)
	{
		delete item;
		return -1;
	}
	for (int i = 0; text[i]; ++i)
	{
		item->text[i] = text[i];
	}
	item->text[strlen(text)] = 0;
	item->def = def;
	NSString* tstr = [NSString stringWithUTF8String:item->text];
	item->instance = [macStatusBar addMenuItem:tstr]; 
	[tstr release];

	TrayPopupMenuItem** nextp = &popupMenu;

	while (*nextp)
		nextp = &((*nextp)->next);
	*nextp = item;

	return 0;
}

bool MacTrayIcon::handleEvent(void* evt)
{
	for (TrayPopupMenuItem* it = popupMenu; it; it = it->next)
	{
		if (it->instance == evt)
		{
			listener->onAction(it->id);
			return true;
		}
	}
	return false;
}

int MacTrayIcon::getSystemSocket()
{
	return mac_pipe[0];
}
