#ifndef TRAYICON_H
#define TRAYICON_H

class TrayIconListener
{
public:
	virtual ~TrayIconListener(){}

	virtual void onAction(int id) = 0;
	virtual void onDefaultAction() = 0;
};

class TrayIcon
{
public:
	virtual ~TrayIcon(){}

	/** Add an icon to the tray icon.
	 * @param icon the icon data, may differ between platforms.
	 * @returns the icon id on success, a number < 0 on error. */
	virtual int addIcon(void* icon) = 0;

	/** Set a specific icon as active. 
	 * @param iconId the iconId to set as active.
	 * @returns 0 on success, non-zero on error. */
	virtual int setActiveIcon(int iconId) = 0;

	/** Show or hide the doc icon. Calling show befor adding an icon will
	 * generate an error.
	 * @param vis 0 means hide the tray icon, 1 means show it. 
	 * @returns 0 on success, non-zero on error. */
	virtual int show(int vis) = 0;

	/** Clear the popup menu so it contains no items. */
	virtual void clearPopupMenu() = 0;

	/** Add a new item to the popup menu.
	 * @returns 0 on success, non-zero on error. */
	virtual int addPopupMenuItem(const char* text, int id, bool def) = 0;
};

#endif // TRAYICON_H

