/* -*- Mode: c++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*-
**
** Copyright 2010 Tim Johansson.  All rights reserved.
**
*/

#define _CRT_SECURE_NO_WARNINGS

#include <stdlib.h>
#include <string.h>
#include "versionchecker.h"
#include "../version.h"

#define VERSION_SERVER "komclean.sourceforge.net"
#define VERSION_SERVER_PORT 80
#define VERSION_FILE "http://komclean.sourceforge.net/latest_version.html"

VersionChecker::VersionChecker() : connecting(false), socket(-1), contentLen(-1), message("")
{}

VersionChecker::~VersionChecker()
{
	disconnect();
}

void VersionChecker::sendRequest()
{
	struct hostent* addr;

	addr = gethostbyname(VERSION_SERVER);
	if (!addr || addr->h_length < 1)
	{
		return;
	}
	socket = ::socket(PF_INET, SOCK_STREAM, 0);
	if (socket < 0)
	{
		return;
	}
#ifdef WIN32
	// AsyncSelect sets the socket in non blocking mode
	WSAAsyncSelect(socket, g_win, WM_SOCKET, FD_READ|FD_CLOSE|FD_CONNECT|FD_WRITE);
#else // !WIN32
	int flags;
#ifdef O_NONBLOCK
	if ((flags = fcntl(socket, F_GETFL, 0)) == -1)
		flags = 0;
	fcntl(socket, F_SETFL, flags | O_NONBLOCK);
#else
	flags = 1;
	ioctl(socket, FIOBIO, &flags);
#endif
#endif // !WIN32
	versrvaddr.sin_family = AF_INET;
	versrvaddr.sin_port = htons(VERSION_SERVER_PORT);
	versrvaddr.sin_addr.s_addr = *(UINT32*)(addr->h_addr_list[0]);
	connecting = true;
	int conres = ::connect(socket, (struct sockaddr*)&versrvaddr, sizeof(versrvaddr));
	if (conres<0)
	{
		if (socket_errno != EINPROGRESS && socket_errno != EAGAIN)
			disconnect();
		return;
	}
	processData();
}

void VersionChecker::processData()
{
	if (connecting)
	{
		// FIXME: Should check the connection status if not win32
		// Write the hello string
		KOMString getreq;
		if (getreq.append("GET "VERSION_FILE" HTTP/1.0\r\n\r\n"))
		{
			disconnect();
			return;
		}
		if (send(socket, getreq.c_string(), getreq.len(), 0) < 0)
		{
			disconnect();
			return;
		}
		connecting = false;
	}
	while (1)
	{
		char tmpreply[1024];
		int replyLen = recv(socket, tmpreply, sizeof(tmpreply)-1, 0);
		if (replyLen > 0)
		{
			tmpreply[replyLen] = 0;
			reply.append(tmpreply, replyLen);
		}
		else
		{
			if (replyLen == 0 || socket_errno != EAGAIN)
			{
				disconnect();
			}
			break;
		}
	}
	const char* endpos = strstr(reply.c_string(), "\r\n\r\n");
	if (contentLen < 0 && endpos)
	{
		// The entire header is loaded
		// Check the status
		int httpminver, status;
		if (!reply.parseLiteral("HTTP/1.") || !reply.parseInt(httpminver) || 
			!reply.parseInt(status) || status != 200)
		{
			disconnect();
			return;
		}
		reply.resetParserPosition();
		// Find the ContentLength: header
		contentLen = 0;
		while (reply.len() >= 2 && (reply.c_string()[0] != '\r' || reply.c_string()[1]!='\n'))
		{
			if (reply.parseLiteral("Content-Length:"))
			{
				if (!reply.parseInt(contentLen))
					contentLen = 0;
			}
			reply.resetParserPosition();
			reply.advanceToAfter('\n');
		}
		reply.advanceToAfter('\n');
		// The remaining is content
		if (contentLen <= 0)
		{
			disconnect();
			return;
		}
	}
	if (contentLen >= 0 && reply.len() >= contentLen)
	{
		// The message is complete
		disconnect();
		// Check if the latest version is newer than current, and if so set the upgrade message
		int newMajor, newMinor, newPatch;
		int curMajor, curMinor, curPatch;
		bool curPre;
		if (!reply.parseInt(newMajor) || !reply.parseLiteral(".") ||
			!reply.parseInt(newMinor) || !reply.parseLiteral(".") ||
			!reply.parseInt(newPatch))
			return;
		reply.resetParserPosition();
		reply.advanceToAfter('\n');
		KOMString curver;
		curver.append(KOMCLEAN_VERSION_STRING);
		if (!curver.parseInt(curMajor) || !curver.parseLiteral(".") ||
			!curver.parseInt(curMinor) || !curver.parseLiteral(".") ||
			!curver.parseInt(curPatch))
			return;
		curver.deleteParsed();
		curPre = curver.len() > 0;
		if (newMajor < curMajor)
			return;
		else if (newMajor == curMajor)
		{
			if (newMinor < curMinor)
				return;
			else if (newMinor == curMinor)
			{
				if (newPatch < curPatch)
					return;
				else if (newPatch == curPatch && !curPre)
					return;
			}
		}
		message = reply.c_string();
	}
}

void VersionChecker::disconnect()
{
	if (socket >= 0)
		close(socket);
	socket = -1;
}

